//===========================================================================//
// Copyright (C) 2011 - DIGITEO - Michael Baudin                             //
// Copyright (C) 2011 - National Institute of Informatics - Benoit Goepfert  //
//                                                                           //
// This file must be used under the terms of the CeCILL.                     //
// This source file is licensed as described in the file COPYING, which      //
// you should have received as part of this distribution.  The terms         //
// are also available at                                                     //
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt                  //
//===========================================================================//

// test 1 : a small matrix (control) (overdetermined)
A=[ -1.    3.    0.    0.    4.    0.  
	 2.   -5.    0.    0.    0.    1.  
	 0.    0.   -2.    3.    0.    0.  
     0.    0.    7.   -1.    0.    0.  
    -3.    0.    0.    4.    6.    0.
	 0.    5.    0.    0.   -7.    8.
	-1     0     0     7     2     0
	 0    -4     3     0     5    -9 ];
spA=sparse(A);
xexp = (1:size(spA,"c"))';
b = spA*xexp; 
trif=1D-3; 
maxiter=3000;
restart=1000;
tol=1.e-6;
x=splspc_rifgmresba(spA,b);
assert_checkalmostequal(x,xexp,0,1D-7);
//
nbiterexp=3;
[x,nbiter,resid]=splspc_rifgmresba(spA,b,trif,restart,maxiter,tol);
assert_checkequal(nbiter,nbiterexp);
assert_checktrue(resid < 1.e-6);
assert_checkalmostequal(x,xexp,0,1D-7);
//
[x2,nbiter2,resid2]=splspc_rifgmresba(spA,b,[],[],[],[]);
assert_checkequal(nbiter,nbiter2);
assert_checkequal(resid,resid2);
assert_checkequal(x,x2);
//
[x3,nbiter3,resid3]=splspc_rifgmresba(spA,b);
assert_checkequal(nbiter,nbiter3);
assert_checkequal(resid,resid3);
assert_checkequal(x,x3);

//test 2 : a large sparse matrix, illc1850
path = fullfile(splspc_getpath(),"tests","matrices");
filename = fullfile(path,"illc1850.mtx");
spA = mmread(filename);
m = size(spA,1);
b = rand(m,1);

[x,nbiter,resid] = splspc_rifgmresba(spA,b);
assert_checktrue(nbiter < maxiter);
assert_checktrue(resid < tol);

[x2,nbiter2,resid2] = splspc_gmresba(spA,b);
assert_checktrue(nbiter < nbiter2);
assert_checkalmostequal(norm(b-spA*x),norm(b-spA*x2),1.e-5);

x3=spA\b;
assert_checkalmostequal(norm(b-spA*x),norm(b-spA*x3),1.e-5);

//breakdown (in preconditioning process)
A=[1 2;1 2;1 2];
spA=sparse(A);
xexp = (1:size(spA,"c"))';
b = spA*xexp; 
name="splspc_rifgmresba";
msg1="%s: preconditioning process breaks down because of non-full rank matrix \n";
assert_checkerror("[x,nbiter,resid]=splspc_rifgmresba(spA,b)", msg1, [], name);
